#Reboots Exchange servers for patching, accommodating roles, HA, and database copies.
#v2.4, 10/30/2012

[CmdletBinding(DefaultParameterSetName='env')]   
Param (
	[Parameter(ParameterSetName='env')][ValidateSet('Production','DR','All')][string]$Environment = 'All',
	[Parameter(ParameterSetName='server')][ValidateNotNullOrEmpty()][string[]]$ComputerName,
	[ValidateNotNullOrEmpty()][string]$RemotingServer,
	[switch]$RebootOnly,
	[switch]$PauseBeforePatching,
	[string[]]$ReportRecipients,
	[string[]]$StatusRecipients
	)

#Region Variables
#Server groupings
[array]$ProdHC = 'prodhc01','prodhc02' #Production HT/CAS
[array]$ProdUM = 'produ01','produ02' #Production UM
[array]$ProdMB = 'prodm03','prodm04','prodm02','prodm01' #Production MB
[array]$DRHC = 'drhc01','drhc02' #DR HT/CAS
[array]$DRUM = 'dru01' #DR UM
[array]$DRMB = 'drm01','drexm02' #DR MB

#SCOM settings
$bUpdateSCOM = $true #Set to false if SCOM is not used or servers should not be put into maintenance mode
$rmsName = 'rmsserver' #SCOM RMS to connect to if SCOM update is true
$maintModeDuration = 90 #Maximum duration, in minutes, of SCOM maintenance window

$WUCompDelay = 90 #Maximum minutes allowed for Windows Update to complete
$rebootMaxWait = 20 #Maximum minutes allowed for server to complete reboot
$serviceStartDelay = 3 #Minutes to wait for Exchange services to start after reboot
$serviceStartAttempts = 3 #Number of times to check for services started
$transcriptLog = 'c:\scripts\RebootScriptLog.txt' #Transcript file to record screen output and send in email

#Mail settings
$mailSender = 'Server Reboot Script <srs@company.com>'
$mailServer = 'smtp.company.com' #SMTP FQDN to relay through
if (-not($ReportRecipients))
	{
	[array]$ReportRecipients = 'user@company.com' #Default recipients for summary report and log file
	}
$bSendStatusUpdates = $true #Set to false to not send status notification when each server is starting/ending
if ($StatusRecipients) #Automatically enable status notifications if address specified as parameter
	{
	$bSendStatusUpdates = $true
	}
else
	{
	[array]$StatusRecipients = 'phonenumber@smsdomain.com' #Default mobile recips for in-progress status updates
	}

#EndRegion Variables

#Region Functions
function WriteTime
	{
	$a = Get-Date -Format g
	Write-Host "$a`: " -NoNewline
	}

function ValidateServer ($ComputerName)
	{
	$serverList = $ProdHC+$ProdUM+$ProdMB+$DRHC+$DRMB+$DRUM
	$badServer = @()
	foreach ($computer in $ComputerName)
		{
		if (-not([regex]::IsMatch($serverList,$computer,1))) #Case-insenstive search for matching server name
			{
			$badServer += $computer
			}
		}
	if ($badServer.Length -ge 1)
		{
		WriteTime;Write-Host "The following server(s) specified does not match a server defined in the server groupings variables: $($badServer -join ',')" -ForegroundColor Red
		WriteTime;Write-Host "Aborting script." -ForegroundColor Red
		return $false
		}
	return $true
	}
	
function GetExchangeServers
	{
	$configNC=([ADSI]"LDAP://RootDse").configurationNamingContext
	$search = new-object DirectoryServices.DirectorySearcher([ADSI]"LDAP://$configNC")
	$search.Filter = "(&(objectClass=msExchExchangeServer)(versionNumber>=1937801568))"
	$search.PageSize=1000
	$search.PropertiesToLoad.Clear()
	[void] $search.PropertiesToLoad.Add("networkaddress")
	$search.FindAll()
	}

function GetServerFqdnFromNetworkAddress($server)
	{
   $server.properties["networkaddress"] |
      where {$_.ToString().StartsWith("ncacn_ip_tcp")} | %{$_.ToString().SubString(13)}
	}

function ConnectToExchange($server)
	{
	try
		{
		Import-PSSession (New-PSSession -ConfigurationName Microsoft.Exchange -ConnectionUri "http://$server/powershell" -Name 'RemoteExchange') -AllowClobber -DisableNameChecking -ErrorAction Stop | Out-Null
		$script:bConnectSuccess = $true
		}
	catch
		{
		$script:bConnectSuccess = $false
		}
	}
	
function CountDown($waitMinutes,$activity,$status)
	{
	$startTime = Get-Date
	$endTime   = $startTime.addMinutes($waitMinutes)
	$timeSpan = New-Timespan $startTime $endTime
	while ($timeSpan -gt 0)
		{
    	$timeSpan = New-Timespan $(get-date) $endTime
		Write-Progress -Activity $activity -Status $status -SecondsRemaining $($timeSpan.TotalSeconds)
	    Sleep 1
		}
	}
 
function SendNotification($subject,$details,$server)
	{
	$mailSubject = $subject
	$mailBody = "$server: $details"
	Stop-Transcript
	$global:transcriptIsRunning = $false
	Send-MailMessage -Body $mailBody -BodyAsHtml -From $mailSender `
		-SmtpServer $mailServer -Subject $mailSubject -To $ReportRecipients `
		-Attachments $transcriptLog
	}
	
function SendStatus($details,$server)
	{
	$mailSubject = "$server status update"
	$mailBody = "$server: $details"
	$smtp = New-Object Net.Mail.SmtpClient($mailServer)
	$smtp.Send($mailSender,$StatusRecipients,$mailSubject,$mailBody)
	}

function AbortScript($iReason,$server)
	{
	switch ($iReason)
		{
		0 {$reason = 'Could not connect to any Exchange server in organization.'}
		1 {$reason = 'Initial connectivity test failed.'}
		2 {$reason = "Windows Update did not complete within $WUCompDelay minutes."}
		3 {$reason = "Server did not reboot within $rebootMaxWait minutes."}
		4 {$reason = 'Server rebooted, but not all required services started.'}
		5 {$reason = 'Unable to launch Windows Update script.'}
		}
	SendNotification $reason "Server reboot script has aborted" $server
	SendStatus $reason $server
	$Host.UI.RawUI.WindowTitle = $savedTitle
	exit
	}

function LoadSCOMShell
	{
	WriteTime;Write-Host 'Creating remote session to SCOM server...'
	$ErrorActionPreference = 'Stop'
	try 
		{
		$global:scomSession = New-PSSession -ComputerName $rmsName
		Invoke-Command -ScriptBlock {Add-PSSnapin 'Microsoft.EnterpriseManagement.OperationsManager.Client'} -Session $global:scomSession
		Invoke-Command -ScriptBlock {Set-Location 'OperationsManagerMonitoring::'} -Session $global:scomSession
		Invoke-Command -ScriptBlock {param($rmsName) New-ManagementGroupConnection -connectionString:$rmsName | Out-Null} -Session $global:scomSession -ArgumentList $rmsName
		Invoke-Command -ScriptBlock {param($rmsName) Set-Location $rmsName} -Session $global:scomSession -ArgumentList $rmsName
		Invoke-Command -ScriptBlock {$monClass = Get-MonitoringClass | Where-Object {$_.DisplayName -eq 'Windows Server'}} -Session $global:scomSession
		WriteTime;Write-Host "Successfully created remote session to SCOM server." -ForegroundColor Green
		$global:bSCOMSessionOpen = $true
		}
	catch
		{
		WriteTime;Write-Host 'Error: Unable to create remote session to SCOM server...' -ForegroundColor Red
		$global:bSCOMSessionOpen = $false
		}
	$ErrorActionPreference = 'Continue'
	}

function UnloadSCOMShell
	{
	if ($global:bSCOMSessionOpen)
		{
		WriteTime;Write-Host 'Closing remote session to SCOM server...'
		Remove-PSSession $global:scomSession
		$global:bSCOMSessionOpen = $false
		}
	}
	
function StartSCOMMaintenance($nodeName)
	{
	WriteTime;Write-Host 'Putting server into maintenance mode in SCOM...'
	$ErrorActionPreference = 'Stop'
	try
		{
		Invoke-Command -ScriptBlock {param($nodeName) $serverCrit = "DisplayName like '$nodeName%'"} -ArgumentList $nodeName -Session $global:scomSession
		Invoke-Command -ScriptBlock {$monObj = Get-MonitoringObject -MonitoringClass $monClass -Criteria $serverCrit} -Session $global:scomSession
		Invoke-Command -ScriptBlock {param($duration) $now = Get-Date;$maintStartTime = $now.ToUniversalTime();$maintEndTime = $now.AddMinutes($duration).ToUniversalTime()} -ArgumentList $maintModeDuration -Session $global:scomSession
		Invoke-Command -ScriptBlock {New-MaintenanceWindow -MonitoringObject $monObj -StartTime $maintStartTime -EndTime $maintEndTime -Reason PlannedApplicationMaintenance -Comment 'Patch and reboot'}  -Session $global:scomSession
		WriteTime;Write-Host 'Server successfully put into maintenance mode in SCOM.'
		$script:bMaintModeActive = $true
		}
	catch
		{
		WriteTime;Write-Host 'Warning: Server could not be put into maintenance mode in SCOM.' -ForegroundColor Yellow
		$script:bMaintModeActive = $false
		}
	$ErrorActionPreference = 'Continue'
	}

function StopSCOMMaintenance($nodeName)
	{
	WriteTime;Write-Host 'Taking server out of maintenance mode in SCOM...'
	$ErrorActionPreference = 'Stop'
	try
		{
		Invoke-Command -ScriptBlock {param($nodeName) $serverCrit = "DisplayName like '$nodeName%'"} -ArgumentList $nodeName -Session $global:scomSession
		Invoke-Command -ScriptBlock {$monObj = Get-MonitoringObject -MonitoringClass $monClass -Criteria $serverCrit} -Session $global:scomSession
		Invoke-Command -ScriptBlock {$now = Get-Date;$maintEndTime = $now.AddMinutes(1).ToUniversalTime()} -Session $global:scomSession
		Invoke-Command -ScriptBlock {
			$maintWindow = Get-MaintenanceWindow -MonitoringObject $monObj -ErrorAction SilentlyContinue
			if ($maintWindow)
				{
				Set-MaintenanceWindow -MonitoringObject $monObj -EndTime $maintEndTime
				}
			}  -Session $global:scomSession
		WriteTime;Write-Host 'Server successfully taken out of maintenance mode in SCOM.'
		$script:bMaintModeActive = $false
		}
	catch
		{
		WriteTime;Write-Host 'Warning: Error taking server out of maintenance mode in SCOM.' -ForegroundColor Yellow
		$script:bMaintModeActive = $false
		}
	$ErrorActionPreference = 'Continue'
	}
	
function EventMonitor($id,$server,$time)
	{
	do
		{
		try
			{
			$serverLogs = [System.Diagnostics.EventLog]::GetEventLogs($server)
			$appLog = $serverLogs[0]
			$iEntryOffset = 1
			$entries = @()
			do #Build list of log entries since start time
				{
				$entry = $appLog.Entries[($appLog.Entries.Count - $iEntryOffset)]
				$entries += $entry
				$iEntryOffset ++
				}
			until ($entry.TimeWritten -le $time)
			foreach ($line in $entries) #Look for entry that WU has finished
				{
				if (($line.Source -eq 'UCTeam Scripts') -and ($line.InstanceId -eq $id))
					{
					return $true
					}
				}
			WriteTime;Write-Host "$server`: Windows Update still running.  Sleeping 3 minutes."	
			}
		catch
			{
			#Some hotfixes cause network loss while being installed
			WriteTime;Write-Host "$server`: Server inaccessible.  Assuming Windows Update still running.  Sleeping 3 minutes."
			}
		finally
			{
			Countdown 3 "$server`: Windows Update still running." 'Checking again in 3 minutes'
			}
		}
	until ($time.AddMinutes($WUCompDelay) -le (Get-Date)) #Wait no more than max minutes set in WUCompDelay variable
	return $false
	}

function PatchServer($pServer)
	{
	WriteTime;Write-Host "Calling patch job on $server..."
	#Verify connectivity
	if (-not(Test-Path "\\$pServer\c$"))
		{
		WriteTime;Write-Host "$pServer is not accessible. Aborting..." -ForegroundColor Red
		AbortScript 1 $pServer
		}
	#Trigger WU script on remote computer
	$startTime = Get-Date
	WriteTime;try{Invoke-Command -Computer $pServer -ScriptBlock {schtasks /run /tn "Run-WindowsUpdate"} -ErrorAction Stop} `
		catch{AbortScript 5 $pServer}
	#Watch for Windows Update completion
	WriteTime;Write-Host 'Waiting for 2 minutes before monitoring WU completion...'
	Countdown 2 "$server`: Windows Update initiated." 'Allowing 2 minutes before checking status'
	WriteTime;Write-Host "$pserver`: Beginning check for Windows Update completion (no later than $($startTime.AddMinutes($WUCompDelay).ToShortTimeString()))..."
	$eventResult = EventMonitor 1010 $pServer $startTime
	if (-not($eventResult))
		{
		WriteTime;Write-Host "Patch process on $pServer has not completed within allotted time. Aborting..." -ForegroundColor Red
		AbortScript 2 $pServer
		}
	WriteTime;Write-Host "Patch process on $pServer is complete." -ForegroundColor Green
	}

function RebootServer($rServer)
	{
	WriteTime;Write-Host "Rebooting $server..."
	Restart-Computer -ComputerName $rServer -Confirm:$false -Force
	}

function RebootMonitor($machine)
	{
	$j = 0
	$jMax = $rebootMaxWait
	do
		{  #Check uptime and wait until a value less than 5 minutes uptime as 'reboot finished' test
		$j ++
		Countdown 1 "$machine`: Reboot Monitor" '1 minute loop'
		$ErrorActionPreference = "Stop"
        try
			{
			$uptime = (Get-WmiObject -computer $machine -class Win32_PerfFormattedData_PerfOS_System).SystemUptime
			if ($uptime -gt 600)
				{
				WriteTime;Write-Host "Check $j ($jMax max): $machine has not rebooted yet."
				}
			else
				{
				WriteTime;Write-Host "Check $j ($jMax max): $machine is online." -ForegroundColor Green
				}
            }
		catch
			{
			WriteTime;Write-Host "Check $j ($jMax max): $machine is not online yet." -ForegroundColor Yellow
			}
        if ($j -eq $jMax)
			{ #Loop has occurred more than max time configured
			WriteTime;Write-Host "Check maximum ($jMax) has been reached.  Aborting..." -ForegroundColor Red
			return 1
			}
		}
	until (($uptime -lt 600) -and ($uptime -gt 0))
	
	#Once rebooted, check for Exchange services status

	$k = 0
	do
		{
		WriteTime;Write-Host "Waiting $serviceStartDelay minutes for services to start on $machine..."
		Countdown $serviceStartDelay "$machine`: Reboot Monitor" 'Waiting to check for service startup'
		$servicesOK = $true
		$k ++
		WriteTime;Write-Host "Checking required services on $machine..."
		[array]$serviceStatus = Test-ServiceHealth -Server $machine
		foreach ($role in $serviceStatus)
			{
			if ($role.ServicesNotRunning.Count -gt 0)
				{
				$servicesOK = $false
				}
			}
		if ($servicesOK) #All required services are started
			{
			WriteTime;Write-Host "All required services started on $machine." -ForegroundColor Green
			return 0
			}
		WriteTime;Write-Host "One or more required services have not started on $machine." -ForegroundColor Yellow
		}
	until ($k -eq $serviceStartAttempts)
	WriteTime;Write-Host "One or more required services have not started on $machine. Aborting..." -ForegroundColor Red
	return 2 #Required service has not started
	}

function Pause
	{
	WriteTime;Write-Host "Script is paused. Press 'c' to continue when you are ready." -ForegroundColor Yellow
	do
		{
		$key = $Host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
		}
	until ($key.VirtualKeyCode -eq 67)
	}

function ProcessServerGroup($serverGroup,$groupType)
	{
	if ($serverGroup.Length -ge 1)
		{
		foreach ($server in $serverGroup)
			{
			Write-Host "*******************Beginning processing of $server.*******************"
			$Host.UI.RawUI.WindowTitle = "Processing $server"
			if ($bSendStatusUpdates)
				{
				SendStatus "Processing beginning" $server
				}
			if ($bUpdateSCOM -and $global:bSCOMSessionOpen)
				{
				StartSCOMMaintenance $server
				}
			if ($groupType -eq 'MB') #Switch over databases and suspend replication
				{
				$dbases = Get-MailboxDatabaseCopyStatus -Server $server
				$mountedDB = $dbases | where {$_.ActiveCopy -eq $true}
				if ($mountedDB)#Perform switchover of active copies
					{
					WriteTime;Write-Host "Moving active databases off of $server..."
					Move-ActiveMailboxDatabase -Server $server -Confirm:$false | Out-Null
					}
				else
					{
					WriteTime;Write-Host "No active databases on $server."
					}
				#Suspend replication
				WriteTime;Write-Host "Suspending replication of database copies on $server..."
				$dbases | %{
					if ((Get-MailboxDatabase $($_.DatabaseName)).ReplicationType -eq 'Remote') #More than one copy exists
						{
						Suspend-MailboxDatabaseCopy $($_.Name) -SuspendComment 'Server being patched and rebooted' -Confirm:$false
						}
					}
				#Pause the cluster node
				WriteTime;Write-Host "Pausing the cluster node on mailbox server $server..."
				$nodePaused = $false
				try
					{
					Invoke-Command -ScriptBlock {param ($server) cluster.exe node $server /pause | Out-Null} -ArgumentList $server -ComputerName $server -ErrorAction Stop
					$nodePaused = $true
					}
				catch
					{
					WriteTime;Write-Host "There was an error pausing the cluster node.  Continuing anyway." -ForegroundColor Yellow
					}
				}
		
			#Pause script if selected
			if ($PauseBeforePatching)
				{
				Pause
				}
						
			#Patch server
			if ($RebootOnly)
				{
				#Optional code
				}
			else
				{
				PatchServer $server
				}
			
			#Reboot server
			RebootServer $server

			#Monitor reboot
			$rebootStatus = RebootMonitor $server
			if ($rebootStatus -eq 1)
				{
				AbortScript 3 $server
				}
			if ($rebootStatus -eq 2)
				{
				AbortScript 4 $server
				}
			
			WriteTime;Write-Host "Reboot of $server completed successfully."
			
			if ($groupType -eq 'MB') #Resume replication
				{
				#Resume the cluster node if successfully paused before
				if ($nodePaused)
					{
					WriteTime;Write-Host "Resuming the cluster node on mailbox server $server..."
					try
						{
						Invoke-Command -ScriptBlock {param ($server) cluster.exe node $server /resume | Out-Null} -ArgumentList $server -ComputerName $server -ErrorAction Stop
						}
					catch
						{
						WriteTime;Write-Host "There was an error resuming the cluster node.  Continuing anyway." -ForegroundColor Yellow
						}
					}
				
				WriteTime;Write-Host "Waiting 3 minutes for all databases on $server to mount..."
				##Future: Add code to confirm databases mounted##
				Countdown 3 "$server`: Database Mounting" 'Allow time for databases to mount'
				WriteTime;Write-Host "Resuming replication of database copies on $server..."
				$dbases | %{
					if ((Get-MailboxDatabase $($_.DatabaseName)).ReplicationType -eq 'Remote') #More than one copy exists
						{			
						Resume-MailboxDatabaseCopy $($_.Name) -Confirm:$false | Out-Null
						}
					}
			
				#Wait 5 minutes to allow logs to be copied
				WriteTime;Write-Host "Waiting 5 minutes for replication to get caught up..."
				##Future: Check for copy queue length and replay queue length##
				Countdown 5 "$server`: Replication Resumption" 'Allow time for logs to be copied'
				}
			
			if ($bUpdateSCOM -and $global:bSCOMSessionOpen -and $script:bMaintModeActive)
				{
				StopSCOMMaintenance $server
				}
				
			Write-Host "*******************Ending processing of $server.*******************"	
			if ($bSendStatusUpdates)
				{
				SendStatus "Processing complete" $server
				}
			}
		}
	}
	
function BalanceDB
	{
	#Balance active copies so each DB is mounted on server whose activation preference is 1
	WriteTime;Write-Host "Balancing active database copies based on activation preference..."
	$dbs = Get-MailboxDatabase -Status
	foreach ($db in $dbs)
		{
		if ($db.ActivationPreference.GetType().BaseType -eq [System.Array])
			{
			$normalizedActPref = ($db.ActivationPreference | Where-Object {$_.Value -eq 1}).Key.Name
			}
		else
			{
			$primaryActPref = $db.ActivationPreference[0]
			$normalizedActPref = $primaryActPref.Substring(1,$primaryActPref.IndexOf(',')-1)
			}
		$mountedOnServer = $db.MountedOnServer.Substring(0,$db.MountedOnServer.IndexOf('.'))
		if ($mountedOnServer -ne $normalizedActPref)
			{
			WriteTime;Write-Host "Moving active database copy ($($db.Name))..."
			Move-ActiveMailboxDatabase $db.Name -ActivateOnServer $normalizedActPref -Confirm:$false | Out-Null
			}
		}
	WriteTime;Write-Host 'Database balancing complete.' -ForegroundColor Green
	}

#EndRegion Functions	
	
#Region Script Body
#Validate server names if specified as a parameter
if ($PSCmdlet.ParameterSetName -eq 'server')
	{
	if (-not(ValidateServer $ComputerName))
		{exit}
	}

$savedTitle = $Host.UI.RawUI.WindowTitle #Save starting window title

#Check if running interactively
if (-not [Environment]::UserInteractive)
	{$PauseBeforePatching = $false} #Do not allow pausing if not running interactively

#Start transcript for sending log
if (!($global:transcriptIsRunning))
	{
	Start-Transcript -Path $transcriptLog -Append
	$global:transcriptIsRunning = $true
	}

#Connect to Exchange
if ($RemotingServer) #Use specified remoting server instead of random server
	{
	if (-not(ValidateServer [String[]]$RemotingServer)) #Validate Exchange server for remoting
		{exit}
	else
		{
		[array]$serverFQDNs = $RemotingServer
		}
	}
else
	{
	$ExchangeServers = GetExchangeServers
	$serverFQDNs = $ExchangeServers | ForEach-Object {GetServerFqdnFromNetworkAddress $_}
	}

for ($i = 0; $i -lt $serverFQDNs.Length; $i++) #Loop through servers until connection is successful
	{
	ConnectToExchange $serverFQDNs[$i]
	if ($bConnectSuccess)
		{break}
	}
if (-not($bConnectSuccess)) #Abort if unable to connect to any server
	{
	AbortScript 0 $null
	}

	
#Connect to SCOM if enabled
if ($bUpdateSCOM)
	{
	WriteTime;Write-Host 'Option: SCOM maintenance mode support enabled.'
	LoadSCOMShell
	}

if ($RebootOnly) #Note that patching will not be performed
	{
	WriteTime;Write-Host 'Option: RebootOnly switch specified.  Servers will not be patched before rebooting.'
	}
	
#Loop through servers to process
if ($PSCmdlet.ParameterSetName -eq 'env') #Global or prod/DR groups
	{
	if ($Environment -eq 'All' -or $Environment -eq 'Production')
		{
		ProcessServerGroup $ProdHC 'HTCAS'
		ProcessServerGroup $ProdUM 'UM'
		ProcessServerGroup $ProdMB 'MB'
		BalanceDB
		}
	if ($Environment -eq 'All' -or $Environment -eq 'DR')
		{
		ProcessServerGroup $DRHC 'HTCAS'
		ProcessServerGroup $DRUM 'UM'
		ProcessServerGroup $DRMB 'MB'
		}
	}
else #Individual servers specified
	{
	$serverHash = @{ #Hash table for determining role group of specified servers
		'ProdHC' = [string]::Join(',',$ProdHC)
		'ProdUM' = [string]::Join(',',$ProdUM)
		'ProdMB' = [string]::Join(',',$ProdMB)
		'DRHC' = [string]::Join(',',$DRHC)
		'DRUM' = [string]::Join(',',$DRUM)
		'DRMB' = [string]::Join(',',$DRMB)
		}
		
	#Reset server grouping arrays to accommodate specified servers to reboot
	$ProdHC = @();$ProdUM = @();$ProdMB = @();$DRHC = @();$DRUM = @();$DRMB = @()
	
	foreach ($computer in $ComputerName)
		{
		$role = $serverHash.Keys | Where-Object {$serverHash.$_ -like "*$computer*"} #Find which role the server has been designated
		Set-Variable -Name $role -Value ((Get-Variable -Name $role -ValueOnly) + $computer) #Add server to role variable
		}

	#Process server groups, but only groups that contain a specified server will have any action
	ProcessServerGroup $ProdHC 'HTCAS'
	ProcessServerGroup $ProdUM 'UM'
	ProcessServerGroup $ProdMB 'MB'
	if ($ProdMB.Length -ge 1) #Only balance databases if production MB server is included
		{BalanceDB}
	ProcessServerGroup $DRHC 'HTCAS'
	ProcessServerGroup $DRUM 'UM'
	ProcessServerGroup $DRMB 'MB'
	}
	
#Disconnect from SCOM, which only has any effect if it was connected in the first place
UnloadSCOMShell

#Close remote Exchange session
Remove-PSSession -Name 'RemoteExchange'

#Final notification
SendNotification 'Patch and reboot script finished successfully.' 'All servers have been rebooted.' 'All Servers'
$Host.UI.RawUI.WindowTitle = $savedTitle #Return window title to before script run
#EndRegion Script Body

<#
.SYNOPSIS
Patches and reboots the Exchange servers in an environment.

.DESCRIPTION
Patches and reboots any or all of the Exchange servers in the enterprise, 
allowing for production and DR environments.  Using server roles, high
availability is maintained, including database replication in a DAG.  Optionally,
patching can be skipped and the servers simply rebooted.  Includes email and
text messaging notification (via SMTP) and SCOM support.

.PARAMETER Environment
Specify which server environment (Production, DR, All) to process.  The default is All.

.PARAMETER ComputerName
Specify one or more server names (comma-delimited) to process.  Names will be 
validated against the list of servers defined in the Server Groupings variable
section.

.PARAMETER RemotingServer
Specify an Exchange server to use for remoting when running Exchange cmdlets.  If not 
specified, a random server will be selected from the list provided by AD.

.PARAMETER RebootOnly
Switch to indicate that patching should be skipped and the servers simply rebooted.

.PARAMETER PauseBeforePatching
Switch to indicate that the script should pause before the patch script is called (if 
applicable).  This allows for manual action on the server, such as running a rollup 
installation outside of the script.  The switch will be forcibly set to false if 
the script is running non-interactively.

.PARAMETER ReportRecipients
Specify one or more email addresses (comma-delimited) to receive the summary report
and log file.  If this parameter is used, it will override the address(es) specified
in the variables region.

.PARAMETER StatusRecipients
Specify one or more email addresses (comma-delimited), usually text message addresses, 
to receive status updates during the script run.  If this parameter is used, it will 
override the address(es) specified in the variables region, and will automatically enable 
the option to receive status updates.

.EXAMPLE
.\Reboot-ExchangeServers.ps1

.EXAMPLE
.\Reboot-ExchangeServers.ps1 -RebootOnly

.EXAMPLE
.\Reboot-ExchangeServers.ps1 -RebootOnly -PauseBeforePatching

.EXAMPLE
.\Reboot-ExchangeServers.ps1 -Environment DR -ReportRecipients address1@company.com,address2@company.com

.EXAMPLE
.\Reboot-ExchangeServers.ps1 -ComputerName Computer1,Computer2 -RebootOnly

.NOTES
The Environment and ComputerName parameters are mutually exclusive, i.e., 
you cannot use both at the same time.  The implicit parameter declaration is 
Environment with a value of All.  The RebootOnly switch can be used with either 
parameter.
#>